@@ -1,13 +1,12 @@
 //! This module contains conversions between various Rust object and their representation in Python.
 
 pub mod anyhow;
-mod array;
 pub mod chrono;
 pub mod eyre;
 pub mod hashbrown;
 pub mod indexmap;
 pub mod num_bigint;
 pub mod num_complex;
-mod osstr;
-mod path;
+pub mod rust_decimal;
 pub mod serde;
+mod std;
