@@ -3,11 +3,14 @@
 use std::convert::TryInto;
 
 use crate::ffi::{self, Py_ssize_t};
+#[cfg(feature = "experimental-inspect")]
+use crate::inspect::types::TypeInfo;
 use crate::internal_tricks::get_ssize_index;
+use crate::types::PyList;
 use crate::types::PySequence;
 use crate::{
     exceptions, AsPyPointer, FromPyObject, IntoPy, IntoPyPointer, Py, PyAny, PyErr, PyObject,
-    PyResult, PyTryFrom, Python, ToPyObject,
+    PyResult, Python, ToPyObject,
 };
 
 #[inline]
@@ -59,7 +62,7 @@ impl PyTuple {
     ///
     /// If you want to create a [`PyTuple`] with elements of different or unknown types, or from an
     /// iterable that doesn't implement [`ExactSizeIterator`], create a Rust tuple with the given
-    /// elements and convert it at once.
+    /// elements and convert it at once using `into_py`.
     ///
     /// # Examples
     ///
@@ -119,7 +122,7 @@ impl PyTuple {
 
     /// Returns `self` cast as a `PySequence`.
     pub fn as_sequence(&self) -> &PySequence {
-        unsafe { PySequence::try_from_unchecked(self) }
+        unsafe { self.downcast_unchecked() }
     }
 
     /// Takes the slice `self[low:high]` and returns it as a new tuple.
@@ -144,7 +147,7 @@ impl PyTuple {
     /// # fn main() -> PyResult<()> {
     /// Python::with_gil(|py| -> PyResult<()> {
     ///     let ob = (1, 2, 3).to_object(py);
-    ///     let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+    ///     let tuple: &PyTuple = ob.downcast(py).unwrap();
     ///     let obj = tuple.get_item(0);
     ///     assert_eq!(obj.unwrap().extract::<i32>().unwrap(), 1);
     ///     Ok(())
@@ -211,6 +214,15 @@ impl PyTuple {
             length: self.len(),
         }
     }
+
+    /// Return a new list containing the contents of this tuple; equivalent to the Python expression `list(tuple)`.
+    ///
+    /// This method is equivalent to `self.as_sequence().to_list()` and faster than `PyList::new(py, self)`.
+    pub fn to_list(&self) -> &PyList {
+        self.as_sequence()
+            .to_list()
+            .expect("failed to convert tuple to list")
+    }
 }
 
 index_impls!(PyTuple, "tuple", PyTuple::len, PyTuple::get_slice);
@@ -291,6 +303,11 @@ macro_rules! tuple_conversion ({$length:expr,$(($refN:ident, $n:tt, $T:ident)),+
                ret
             }
         }
+
+        #[cfg(feature = "experimental-inspect")]
+fn type_output() -> TypeInfo {
+            TypeInfo::Tuple(Some(vec![$( $T::type_output() ),+]))
+        }
     }
 
     impl <$($T: IntoPy<PyObject>),+> IntoPy<Py<PyTuple>> for ($($T,)+) {
@@ -302,12 +319,17 @@ macro_rules! tuple_conversion ({$length:expr,$(($refN:ident, $n:tt, $T:ident)),+
                 ret
             }
         }
+
+        #[cfg(feature = "experimental-inspect")]
+        fn type_output() -> TypeInfo {
+            TypeInfo::Tuple(Some(vec![$( $T::type_output() ),+]))
+        }
     }
 
     impl<'s, $($T: FromPyObject<'s>),+> FromPyObject<'s> for ($($T,)+) {
         fn extract(obj: &'s PyAny) -> PyResult<Self>
         {
-            let t = <PyTuple as PyTryFrom>::try_from(obj)?;
+            let t: &PyTuple = obj.downcast()?;
             if t.len() == $length {
                 #[cfg(any(Py_LIMITED_API, PyPy))]
                 return Ok(($(t.get_item($n)?.extract::<$T>()?,)+));
@@ -318,6 +340,11 @@ macro_rules! tuple_conversion ({$length:expr,$(($refN:ident, $n:tt, $T:ident)),+
                 Err(wrong_tuple_length(t, $length))
             }
         }
+
+        #[cfg(feature = "experimental-inspect")]
+fn type_input() -> TypeInfo {
+            TypeInfo::Tuple(Some(vec![$( $T::type_input() ),+]))
+        }
     }
 });
 
@@ -427,8 +454,8 @@ tuple_conversion!(
 
 #[cfg(test)]
 mod tests {
-    use crate::types::{PyAny, PyTuple};
-    use crate::{PyTryFrom, Python, ToPyObject};
+    use crate::types::{PyAny, PyList, PyTuple};
+    use crate::{Python, ToPyObject};
     use std::collections::HashSet;
 
     #[test]
@@ -450,7 +477,7 @@ mod tests {
     fn test_len() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             assert_eq!(3, tuple.len());
             let ob: &PyAny = tuple.into();
             assert_eq!((1, 2, 3), ob.extract().unwrap());
@@ -472,7 +499,7 @@ mod tests {
     fn test_iter() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             assert_eq!(3, tuple.len());
             let mut iter = tuple.iter();
 
@@ -493,7 +520,7 @@ mod tests {
     fn test_into_iter() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             assert_eq!(3, tuple.len());
 
             for (i, item) in tuple.iter().enumerate() {
@@ -507,7 +534,7 @@ mod tests {
     fn test_as_slice() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
 
             let slice = tuple.as_slice();
             assert_eq!(3, slice.len());
@@ -585,7 +612,7 @@ mod tests {
     fn test_tuple_get_item_invalid_index() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             let obj = tuple.get_item(5);
             assert!(obj.is_err());
             assert_eq!(
@@ -599,7 +626,7 @@ mod tests {
     fn test_tuple_get_item_sanity() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             let obj = tuple.get_item(0);
             assert_eq!(obj.unwrap().extract::<i32>().unwrap(), 1);
         });
@@ -610,7 +637,7 @@ mod tests {
     fn test_tuple_get_item_unchecked_sanity() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             let obj = unsafe { tuple.get_item_unchecked(0) };
             assert_eq!(obj.extract::<i32>().unwrap(), 1);
         });
@@ -620,7 +647,7 @@ mod tests {
     fn test_tuple_index_trait() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             assert_eq!(1, tuple[0].extract::<i32>().unwrap());
             assert_eq!(2, tuple[1].extract::<i32>().unwrap());
             assert_eq!(3, tuple[2].extract::<i32>().unwrap());
@@ -632,7 +659,7 @@ mod tests {
     fn test_tuple_index_trait_panic() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             let _ = &tuple[7];
         });
     }
@@ -641,7 +668,7 @@ mod tests {
     fn test_tuple_index_trait_ranges() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             assert_eq!(vec![2, 3], tuple[1..3].extract::<Vec<i32>>().unwrap());
             assert_eq!(
                 Vec::<i32>::new(),
@@ -661,7 +688,7 @@ mod tests {
     fn test_tuple_index_trait_range_panic_start() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             tuple[5..10].extract::<Vec<i32>>().unwrap();
         })
     }
@@ -671,7 +698,7 @@ mod tests {
     fn test_tuple_index_trait_range_panic_end() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             tuple[1..10].extract::<Vec<i32>>().unwrap();
         })
     }
@@ -681,7 +708,7 @@ mod tests {
     fn test_tuple_index_trait_range_panic_wrong_order() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             #[allow(clippy::reversed_empty_ranges)]
             tuple[2..1].extract::<Vec<i32>>().unwrap();
         })
@@ -692,7 +719,7 @@ mod tests {
     fn test_tuple_index_trait_range_from_panic() {
         Python::with_gil(|py| {
             let ob = (1, 2, 3).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             tuple[8..].extract::<Vec<i32>>().unwrap();
         })
     }
@@ -701,7 +728,7 @@ mod tests {
     fn test_tuple_contains() {
         Python::with_gil(|py| {
             let ob = (1, 1, 2, 3, 5, 8).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             assert_eq!(6, tuple.len());
 
             let bad_needle = 7i32.to_object(py);
@@ -719,7 +746,7 @@ mod tests {
     fn test_tuple_index() {
         Python::with_gil(|py| {
             let ob = (1, 1, 2, 3, 5, 8).to_object(py);
-            let tuple = <PyTuple as PyTryFrom>::try_from(ob.as_ref(py)).unwrap();
+            let tuple: &PyTuple = ob.downcast(py).unwrap();
             assert_eq!(0, tuple.index(1i32).unwrap());
             assert_eq!(2, tuple.index(2i32).unwrap());
             assert_eq!(3, tuple.index(3i32).unwrap());
@@ -797,9 +824,8 @@ mod tests {
 
         impl Clone for Bad {
             fn clone(&self) -> Self {
-                if self.0 == 42 {
-                    panic!()
-                };
+                // This panic should not lead to a memory leak
+                assert_ne!(self.0, 42);
                 NEEDS_DESTRUCTING_COUNT.fetch_add(1, SeqCst);
 
                 Bad(self.0)
@@ -838,10 +864,11 @@ mod tests {
         }
 
         Python::with_gil(|py| {
-            let _ = std::panic::catch_unwind(|| {
+            std::panic::catch_unwind(|| {
                 let iter = FaultyIter(0..50, 50);
                 let _tuple = PyTuple::new(py, iter);
-            });
+            })
+            .unwrap_err();
         });
 
         assert_eq!(
@@ -865,10 +892,8 @@ mod tests {
 
         impl Clone for Bad {
             fn clone(&self) -> Self {
-                if self.0 == 3 {
-                    // This panic should not lead to a memory leak
-                    panic!()
-                };
+                // This panic should not lead to a memory leak
+                assert_ne!(self.0, 3);
                 NEEDS_DESTRUCTING_COUNT.fetch_add(1, SeqCst);
 
                 Bad(self.0)
@@ -890,9 +915,10 @@ mod tests {
         let s = (Bad(1), Bad(2), Bad(3), Bad(4));
         NEEDS_DESTRUCTING_COUNT.store(4, SeqCst);
         Python::with_gil(|py| {
-            let _ = std::panic::catch_unwind(|| {
+            std::panic::catch_unwind(|| {
                 let _tuple: Py<PyAny> = s.to_object(py);
-            });
+            })
+            .unwrap_err();
         });
         drop(s);
 
@@ -902,4 +928,14 @@ mod tests {
             "Some destructors did not run"
         );
     }
+
+    #[test]
+    fn test_tuple_to_list() {
+        Python::with_gil(|py| {
+            let tuple = PyTuple::new(py, vec![1, 2, 3]);
+            let list = tuple.to_list();
+            let list_expected = PyList::new(py, vec![1, 2, 3]);
+            assert!(list.eq(list_expected).unwrap());
+        })
+    }
 }
